﻿using Castle.DynamicProxy;
using Microsoft.Extensions.DependencyInjection;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Hotline.Settings.SystemLogDomain;
public static class SystemLogServiceCollectionExtensions
{
    public static IServiceCollection AddProxiedScoped<TInterface, TImplementation>(this IServiceCollection services)
        where TInterface : class
        where TImplementation : class, TInterface
    {
        services.AddScoped<TImplementation>();
        services.AddScoped<TInterface>(provider =>
        {
            var proxyGenerator = new ProxyGenerator();
            var target = provider.GetRequiredService<TImplementation>();
            var interceptor = provider.GetRequiredService<AsyncLoggingInterceptor>();
            return proxyGenerator.CreateInterfaceProxyWithTarget<TInterface>(target, interceptor);
        });

        return services;
    }

    public static IServiceCollection AddProxiedScoped(this IServiceCollection services, Type serviceType, Type implementationType)
    {
        services.AddScoped(implementationType);
        services.AddScoped(serviceType, provider =>
        {
            var proxyGenerator = new ProxyGenerator();
            var target = provider.GetRequiredService(implementationType);
            var interceptor = provider.GetRequiredService<AsyncLoggingInterceptor>();

            return proxyGenerator.CreateInterfaceProxyWithTarget(serviceType, target, interceptor);
        });

        return services;
    }
}
